#assessment.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#10/07/2006

############################################################
#Assessment / observation model#############################
############################################################

#function calculates new assessment ssb/f by applying a bias 
#and noise to the existing values

assessment<-function(ssbold,
		     fbarold,
		     ssbassessbias,
		     ssbassesscv,
		     fbarassessbias,
		     fbarassesscv){

   #all arguments are required
   #all arguments should be numeric
   #all arguments should be of length 1
   #ssbold should be positive
   #ssbassesscv,fbarassesscv should be positive
   #ssbassessbias, fbarassessbias should be positive
  
   missing.args<-c()

   if (missing(ssbold)) missing.args<-c(missing.args,"ssbold")
   if (missing(fbarold)) missing.args<-c(missing.args,"fbarold")
   if (missing(ssbassessbias)) missing.args<-c(missing.args,"ssbassessbias")
   if (missing(ssbassesscv)) missing.args<-c(missing.args,"ssbassesscv")
   if (missing(fbarassessbias)) missing.args<-c(missing.args,"fbarassessbias")
   if (missing(fbarassesscv)) missing.args<-c(missing.args,"fbarassesscv")

   if (length(missing.args)>0) {
      stop(cat("ffactor::the following required argument(s) are missing:",missing.args,"\n"))
   }

   if (!(is.numeric(ssbold))) stop ("assessment::argument ssbold should be numeric\n")
   if (!(is.numeric(fbarold))) stop ("assessment::argument fbarold should be numeric\n")
   if (!(is.numeric(ssbassessbias))) stop ("assessment::argument ssbassessbias should be numeric\n")
   if (!(is.numeric(ssbassesscv))) stop ("assessment::argument ssbassesscv should be numeric\n")
   if (!(is.numeric(fbarassessbias))) stop ("assessment::argument fbarassessbias should be numeric\n")
   if (!(is.numeric(fbarassesscv))) stop ("assessment::argument fbarassesscv should be numeric\n")

   if (!length(ssbold) == 1) stop ("assessment::argument ssbold should be of length 1\n")
   if (!length(fbarold) == 1) stop ("assessment::argument fbarold should be of length 1\n")
   if (!length(ssbassessbias) == 1) stop ("assessment::argument ssbassessbias should be of length 1\n")
   if (!length(ssbassesscv) == 1) stop ("assessment::argument ssbassesscv should be of length 1\n")
   if (!length(fbarassessbias) == 1) stop ("assessment::argument fbarassessbias should be of length 1\n")
   if (!length(fbarassesscv) == 1) stop ("assessment::argument fbarassesscv should be of length 1\n")

   if (ssbold<0) stop("assessment::argument ssbold must be >= 0\n")
   if (ssbassessbias<0) stop("assessment::argument ssbassessbias must be >= 0\n")
   if (ssbassesscv<0) stop("assessment::argument ssbassesscv must be >= 0\n")

   #if (fbarold<0) stop("assessment::argument fbarold must be >= 0\n")
   if (fbarassessbias<0) stop("assessment::argument fbarassessbias must be >= 0\n")
   if (fbarassesscv<0) stop("assessment::argument fbarassesscv must be >= 0\n")

   #apply bias and cv to ssb assessment
   #ssb

   newssb <- -1

   while (newssb<0) {

      newssb<-rnorm(1,ssbold*ssbassessbias,ssbassesscv*ssbold*ssbassessbias)
   
   }
   
   ssb<-newssb
  
   #apply bias and cv to f assessment
   
   #fbar
   
   newfbar<--1
   
   while (newfbar<0) {

      newfbar<-rnorm(1,fbarold*fbarassessbias,fbarassesscv*fbarold*fbarassessbias)

   }
   
   fbar <- newfbar
  
   #return vector
   assres <- c(ssb,fbar)

}
