#hockeystick.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#10/07/2006

##################################################################################
# Function to calculate hockey stick parameters from historical recruitment data #
# The gradient of the fitted line is returned ####################################
# Inputs are the file containing the recruitment data and the ssb cutoff point####
##################################################################################

hockey<-function(recfile,ssbcutoff){

   if (missing(recfile) & missing(ssbcutoff)) stop ("hockey::recfile and ssbcutoff are manadatory parameters\n")
   if (missing(recfile)) stop ("hockey::recfile is a mandatory parameter\n")
   if (missing(ssbcutoff)) stop ("hockey::ssbcutoff is a mandatory parameter\n")

   #check rec file exists
   if (!(file.exists(recfile))) stop(paste("hockey::",recfile," doesn't exist\n"))

   #ssbcutoff must be numeric, of length 1 and >=0
   if (!(is.numeric(ssbcutoff))) stop("hockey::ssbcutoff should be numeric\n")
   if (!(length(ssbcutoff)==1)) stop("hockey::ssbcutoff should be of length 1\n")
   if (ssbcutoff<0) stop("hockey::ssbcutoff should be >= 0\n")

  #scan recruitment data
  recmatrix <- matrix(scan(recfile),ncol=3,byrow=TRUE)
  
  #subset the matrix to get the values corresponding to an ssb less than or equal to the cut off
  recsub <- subset(recmatrix, recmatrix[,3] <= ssbcutoff)
  
  #perform a least squares analysis to fit a line through the origin to the subset
  y <- recsub[,2]
  x <- recsub[,3]
  lsq <- lm(y~x-1)
  
  #retrieve the coefficients
  z <- coef(lsq)
  
}

