#loadrundata.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie

#load the FLR data objects from a single f/tac simulation

loadrundata <- function(runref="missing",resref="missing"){

   if (missing(runref)) {runref<-winDialogString("Run Reference","")}

   if (nchar(runref)==0) stop("Exit - invalid run reference")

   #construct options filename
   optionsfile = paste(FPRESS.Home(),"\\options\\options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   resolution <- as.integer(nodeval(root,"resolution",0))

   if (missing(resref)) {
      if (resolution > 1) {
         resref <- winDialogString(paste("Resolution Reference (1-",resolution+1,")",sep=""),"")
      } else {
	 resref <- "1"
      }
   }

   FLRFile <- paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",resref,".dat",sep="")

   #check FLR file exists
   if (!(file.exists(FLRFile))) {
      stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   load(FLRFile)

}

