#management.r
#Andrew Campbell
#Marine Institite, Galway
#andrew.campbell@marine.ie
#10/07/2006

############################################################
#Management model###########################################
############################################################

management<-function(ssb,
		     fbar,
		     ftac,
		     yearno,
		     years,
		     assessdelay,
		     hcrperiod,
		     hcrrule,
		     hcrchange,
		     ssbhcr,
		     oldhcrmult,
		     fbarhcr,
		     ssbtest1,
		     ssbtest2,
		     mintac){

   #all arguments are mandatory
   missing.args<-c()

   if (missing(ssb)) missing.args<-c(missing.args,"ssb")
   if (missing(fbar)) missing.args<-c(missing.args,"fbar")
   if (missing(ftac)) missing.args<-c(missing.args,"ftac")
   if (missing(yearno)) missing.args<-c(missing.args,"yearno")
   if (missing(years)) missing.args<-c(missing.args,"years")
   if (missing(assessdelay)) missing.args<-c(missing.args,"assessdelay")
   if (missing(hcrperiod)) missing.args<-c(missing.args,"hcrperiod")
   if (missing(hcrrule)) missing.args<-c(missing.args,"hcrrule")
   if (missing(hcrchange)) missing.args<-c(missing.args,"hcrchange")
   if (missing(ssbhcr)) missing.args<-c(missing.args,"ssbhcr")
   if (missing(oldhcrmult)) missing.args<-c(missing.args,"oldhcrmult")
   if (missing(fbarhcr)) missing.args<-c(missing.args,"fbarhcr")
   if (missing(ssbtest1)) missing.args<-c(missing.args,"ssbtest1")
   if (missing(ssbtest2)) missing.args<-c(missing.args,"ssbtest2")
   if (missing(mintac)) missing.args<-c(missing.args,"mintac")

   if (length(missing.args)>0) {
      stop(cat("management::the following required argument(s) are missing:",missing.args,"\n"))
   }

   #ssb should be numeric, of length 1 and not less than zero
   if (!(is.numeric(ssb))) stop("management::ssb should be numeric\n")
   if (!(length(ssb)==1)) stop("management::ssb should be of length 1\n")
   if (ssb<0) stop("management::ssb should be >= 0\n")

   #fbar should be numeric, of length 1 and not less than zero
   if (!(is.numeric(fbar))) stop("management::fbar should be numeric\n")
   if (!(length(fbar)==1)) stop("management::fbar should be of length 1\n")
   if (fbar<0) stop("management::fbar should be >= 0\n")

   #ftac should be numeric, of length 1 and not less than zero
   if (!(is.numeric(ftac))) stop("management::ftac should be numeric\n")
   if (!(length(ftac)==1)) stop("management::ftac should be of length 1\n")
   if (ftac<0) stop("management::ftac should be >= 0\n")

   #years should be numeric, of length 1 and greater than zero
   if (!(is.numeric(years))) stop("management::years should be numeric\n")
   if (!(length(years)==1)) stop("management::years should be of length 1\n")
   if (years<=0) stop("management::years should be > 0\n")

   #yearno should be numeric, of length 1 and between the values of 1 and years
   if (!(is.numeric(yearno))) stop("management::yearno should be numeric\n")
   if (!(length(yearno)==1)) stop("management::yearno should be of length 1\n")
   if ((yearno<1) || (yearno>years)) stop("management::yearno should be between 1 and ",years,"\n")

   #assessdelay should be numeric, of length 1 and not less than zero
   if (!(is.numeric(assessdelay))) stop("management::assessdelay should be numeric\n")
   if (!(length(assessdelay)==1)) stop("management::assessdelay should be of length 1\n")
   if (assessdelay<0) stop("management::assessdelay should be >= 0\n")

   #hcrperiod should be numeric, of length 1 and greater than or equal to 1
   if (!(is.numeric(hcrperiod))) stop("management::hcrperiod should be numeric\n")
   if (!(length(hcrperiod)==1)) stop("management::hcrperiod should be of length 1\n")
   if (hcrperiod<1) stop("management::hcrperiod should be >= 1\n")

   #hcrrule should be numeric, of length 1 and positive
   if (!(is.numeric(hcrrule))) stop("management::hcrrule should be numeric\n")
   if (!(length(hcrrule)==1)) stop("management::hcrrule should be of length 1\n")
   if (hcrrule<0) stop("management::hcrrule should be >= 0\n")

   #hcrchange should be numeric, of length 1 and between 0 and 1
   if (!(is.numeric(hcrchange))) stop("management::hcrchange should be numeric\n")
   if (!(length(hcrchange)==1)) stop("management::hcrchange should be of length 1\n")   
   if ((hcrchange<0) || (hcrchange>1)) stop("management::hcrchange should be between 0 and 1\n")
   
   #ssbhcr should be numeric, of length 1 and not less than zero
   if (!(is.numeric(ssbhcr))) stop("management::ssbhcr should be numeric\n")
   if (!(length(ssbhcr)==1)) stop("management::ssbhcr should be of length 1\n")
   if (ssbhcr<0) stop("management::ssbhcr should be >= 0\n")

   #oldhcrmult should be numeric and of length 1 and not less than zero
   if (!(is.numeric(oldhcrmult))) stop("management::oldhcrmult should be numeric\n")
   if (!(length(oldhcrmult)==1)) stop("management::oldhcrmult should be of length 1\n")
   if (oldhcrmult<0) stop("management::oldhcrmult should be >= 0\n")

   #fbarhcr should be numeric, of length 1 and not less than zero
   if (!(is.numeric(fbarhcr))) stop("management::fbarhcr should be numeric\n")
   if (!(length(fbarhcr)==1)) stop("management::fbarhcr should be of length 1\n")
   if (fbarhcr<0) stop("management::fbarhcr should be >= 0\n")

   #ssbtest1 should be numeric, of length 1 and not less than zero
   if (!(is.numeric(ssbtest1))) stop("management::ssbtest1 should be numeric\n")
   if (!(length(ssbtest1)==1)) stop("management::ssbtest1 should be of length 1\n")
   if (ssbtest1<0) stop("management::ssbtest1 should be >= 0\n")

   #ssbtest2 should be numeric, of length 1 and not less than zero
   if (!(is.numeric(ssbtest2))) stop("management::ssbtest2 should be numeric\n")
   if (!(length(ssbtest2)==1)) stop("management::ssbtest2 should be of length 1\n")
   if (ssbtest2<0) stop("management::ssbtest2 should be >= 0\n")

   #mintac should be numeric, of length 1 and not less than zero
   if (!(is.numeric(mintac))) stop("management::mintac should be numeric\n")
   if (!(length(mintac)==1)) stop("management::mintac should be of length 1\n")
   if (mintac<0) stop("management::mintac should be >= 0\n")

  #hcr's are not applied unless yearno is greater than the assessdelay
  
  #cat("management : yearno,assessdelay,ftac,ssb",yearno,assessdelay,ftac,ssb,"\n")
  
  if((yearno-assessdelay)==1){oldhcrmult<-1}

  #check is the hcr to be applied in this year?
  #(hcrperiod is the interval between application of the hcr)
  #if the hcr is to be applied the modulus operator applied to yearno and hcrperiod will return 0
  
  yearmod<-yearno%%hcrperiod

  #if yearmod is calculated as NaN, ensure the hcr is not applied
  if(is.na(yearmod)==TRUE){yearmod<-1}

   if(yearmod==0){       #apply the hcr this year

      #default values
      hcrsub <- 0

      #check that a suitable HCR function is available
      if (!exists(paste("hcr",as.character(hcrrule),sep=""))) {
	 stop (paste("\nNo HCR function exists for index ",hcrrule,"\nUnable to continue\n",sep=""))
      }
   
      #calculate the appropriate multiplier for ftac (given the specified hcr)
      if(hcrrule==0){
         hcrmult<-1
	 hcrsub<-0
      }
      
      if(hcrrule==1){hcrmult<-hcr1(ssb,ssbhcr)}
      
      if(hcrrule==2){hcrmult<-hcr2(ssb,ssbhcr)}
      
      if(hcrrule==3){
	 hcrret<-hcr3(ssb,ssbhcr)
         hcrmult<-hcrret[1]
         hcrsub<-hcrret[2]
      }
      
      if(hcrrule==4){hcrmult<-hcr4(fbar,fbarhcr)}
      
      if(hcrrule==5){hcrmult<-hcr5(fbar,fbarhcr)}
      
      if(hcrrule==6){hcrmult<-hcr6(fbar,fbarhcr)}
      
      if(hcrrule==7){
         hcrret<-hcr7(ssb,ssbtest1,ssbtest2)
         hcrmult<-hcrret[1]
         hcrsub<-hcrret[2]
      }

      #USER DEFINED HCRs
      #add extra code here for user-defined harvest control rules
      #insert a suitable if condition, calling the new function in hcr.r
      #and supplying the correct parameters
      #functions should return a multiplier for f/TAC
      #if (hcrrule==8){hcrmult<-hcr8()}

      #limit the change in ftac if the hcrchange parameter is set to a non-zero value
      if (hcrchange>0) {
  
	 #new minimum value
	 newmin <- oldhcrmult*max(c(0,(1-hcrchange)))
      
	 #new maximum value
	 newmax <- oldhcrmult*(1+hcrchange)
     
	 #apply these limits
	 if (hcrmult < newmin) {hcrmult<-newmin}
	 if (hcrmult > newmax) {hcrmult<-newmax}
      }

   } else {      #do not apply hcr and use previous value

      hcrmult <- oldhcrmult  
      hcrsub <- 0
   }

   hcrmult<-c(hcrmult,hcrsub)

   #return the new multiplier
   return(hcrmult)
  
}

