#ssbrisk3dplot.r
#Andrew Campbell
#Marine Institute, Galway
#andrew.campbell@marine.ie

ssbrisk3dplot<-function(runref = "missing",
			ssbtest = "missing"){

   #prompt for run reference
   if (missing(runref)) { 
      runref<-winDialogString("Run Reference?","")
   }

   if (is.null(runref)) {stop("Cancelled\n")}

   if (nchar(runref)==0) stop("Exit - invalid run reference")

   #options file
   optionsfile = paste(FPRESS.Home(),"\\options\\","options",runref,".xml",sep="")

   #check options file exists
   if (!(file.exists(optionsfile))) {
      stop(paste("\nOptions file",optionsfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   #load xml tree
   doc<-xmlTreeParse(optionsfile)

   #root xmlNode object
   root<-xmlRoot(doc)

   #node values
   catchfile <- nodeval(root,"catchfile","")
   startyear <- as.integer(nodeval(root,"startyear","2000"))
   years <- as.integer(nodeval(root,"years",1))
   variable <- as.integer(nodeval(root,"variable",0))
   resolution <- as.integer(nodeval(root,"resolution",0))
   histrec <- as.integer(nodeval(root,"histrec",0))
   histcatch <- as.integer(nodeval(root,"histcatch",0))
   iterations <- as.integer(nodeval(root,"iterations",1))
   varmin <- as.numeric(nodeval(root,"varmin",0))
   varmax <- as.numeric(nodeval(root,"varmax",0))

   if (resolution==0) {stop("Plot cannot be produced for simulations with resolution values of 0\n")}

   if (missing(ssbtest)) {

      ssbtest <- as.numeric(nodeval(root,"ssbtest1",0))
      
      ssbtest <- winDialogString("SSB Test",as.character(ssbtest))

      if (is.null(ssbtest)) {stop("Cancelled\n")}

      ssbtest <- as.numeric(ssbtest)
   }

   if (variable==0) {
      #TAC
      ylabel="TAC"
   } else if (variable==1) {
      #F
      ylabel="F"
   } else {
      #unknown
      varmin<-0
      varmax<-0
   }

   #Historical Catch Data
   #read catch data

   #check catch file exists
   if (!(file.exists(catchfile))) {
      stop(paste("\ncatch data file",catchfile,"doesn't exist\nUnable to continue\n",sep=" "))
   }

   input<-scan(catchfile,list(0,0,0),quiet=TRUE)
    
   catyear<-input[[1]]
   ccatch<-input[[2]]
   cfbar<-input[[3]]

   if(histcatch==1){
     maxcatyr<-max(catyear)
     lencatyr<-length(catyear)
   } else  {
     maxcatyr<-startyear-1
   }

   var<-seq(varmin,varmax,length=resolution+1)
   years<-seq(maxcatyr+1,startyear+years-1,by=1)

   xlabel="Year"
   zlabel="%age Risk"

   yearvector<-c()
   varvector<-c()
   riskvector<-c()

   for (i in 1:(resolution+1)) {

      FLRFile=paste(FPRESS.Home(),"\\outdata\\FLRObjects_",runref,"_",i,".dat",sep="")

      #check FLR file exists
      if (!(file.exists(FLRFile))) {
	 stop(paste("\nFLR data file",FLRFile,"doesn't exist\nUnable to continue\n",sep=" "))
      }

      load(FLRFile)

      ssb<-quantSums(NewSpawnMass)

      varval<-var[i]

      variable<-rep(varval,length(years))

      yearvector<-c(yearvector,years)
      varvector<-c(varvector,variable)

      for (y in 1:length(years)) {

	 yr<-as.character(years[y])

	 yrdata<-as.vector(ssb[,yr,,,])

	 belowssb<-0

	 for (z in 1:iterations) {

	    if(yrdata[[z]]<ssbtest) {
	       belowssb<-belowssb+1
	    }

	 }
	 riskvector<-c(riskvector,100.0*belowssb/iterations)
      }
   }

   final<-data.frame(yearvector,varvector,riskvector)

   wireframe(riskvector~yearvector*varvector,
   data=final,drape=T,screen=list(z=45,x=-60,y=0),
   scales=list(arrows=FALSE),colorkey=FALSE,
   xlab=xlabel,ylab=ylabel,zlab=zlabel,zlim=c(0,100),
   main=paste("Run",runref,"SSB=",ssbtest,sep=" "))

}


